import java.io.*;
import java.lang.Math;
import java.util.*;

/**
 * The <code>Brain</code> class is used to compute and steer the actual
 * player. By receiving all necessary information from the <code>Worldmodel</code>,
 * tactics and player-control are being considered.
 *
 * @author <a href="mailto:ramin@radpour.de">Ramin Radpour</a>
 * @author <a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
 * @author <a href="wini.appl@epost.de">Wini Appl</a>
 * @author <a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 * @author <a href="mailto:alillich@uni-mannheim.de">Alexander Lillich</a>
 * @author <a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
 */
 public class Brain
{
	private Worldmodel world;
	private PlayerAction playerAction;
	private Player player;
	
	private double[][] initPos;	
	private double[][] gamePos;
	
	private int myNr;
	private int actualTime;
	private double[][] positions;
	private double[][] oldPos;
	
	private String refereeMsg = "";
	private String manipulationMessage;
	private String ulukai = "";
	private double[] futurePos;
	
	private boolean ballCatched;
	private boolean playRight;
	private boolean playerChanged;
	
	/**
	*  Creates a new <code>Brain</code> instance.
	*
	*  @param playerAction a <code>PlayerAction</code> value
	*  @param player a <code>Player</code> value
	*  @param world a <code>worldmodel</code> value
	*
	*  @see PlayerAction
	*  @see Player
	*  @see Worldmodel
	*/
	public Brain(PlayerAction playerAction, Player player, Worldmodel world)
	{
		this.playerAction = playerAction;
		this.world = world;
		this.player = player;
		
		playRight = false;
		positions = new double[23][7];
		oldPos = new double[12][3];
		ballCatched = false;
		playerChanged = false;
		manipulationMessage = "";
		initPos();
		gamePos();
	}

	
	/**
	* The <code>initPos</code> method initial positions of all players at start
	* of game or after goal
	*
	* <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	public void initPos()
	{
		initPos = new double[11][2];
		initPos[0][0] = -50; 	initPos[0][1] = 0;
		initPos[1][0] = -35; 	initPos[1][1] = -20;
		initPos[2][0] = -35; 	initPos[2][1] = 0;
		initPos[3][0] = -35; 	initPos[3][1] = 20;
		initPos[4][0] = -20;	initPos[4][1] = -25;
		initPos[5][0] = -20; 	initPos[5][1] = -8;
		initPos[6][0] = -20; 	initPos[6][1] = 8;
		initPos[7][0] = -20; 	initPos[7][1] = 25;
		initPos[8][0] = -5; 	initPos[8][1] = -20;
		initPos[9][0] = -5; 	initPos[9][1] = 0;
		initPos[10][0] = -5; 	initPos[10][1] = 20;
	}
	/**
	* The <code>initStartPos</code> method setting up start positions before 
	* kickoff or after goal
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private void initStartPos()
	{
		playerAction.move(initPos[myNr-1][0], initPos[myNr-1][1]);
	}
	
	/**
	* The <code>gamePos</code> method defines the action-area.
	* Actual positions during the game.
	* <ul>
	* <li> inital Player Positions while playing 
	* <li> gamePos[*][0] = X-Coordinate Player
	* <li> gamePos[*][1] = Y-Coordinate Player
	* <li> gamePos[*][2] = X-Coordinate Player ActionRadius
	* <li> gamePos[*][2] = Y-Coordinate Player ActionRadius
	* </ul>
	*
	* <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	public void gamePos()
	{
		gamePos = new double[11][4];
		gamePos[0][0] = -50; 	gamePos[0][1] = 0;		gamePos[0][2] = 11; 	gamePos[0][3] = 15;
		gamePos[1][0] = -35; 	gamePos[1][1] = -20;	gamePos[1][2] = 17.5;	gamePos[1][3] = 15;
		gamePos[2][0] = -35; 	gamePos[2][1] = 0;		gamePos[2][2] = 17.5;	gamePos[2][3] = 15;
		gamePos[3][0] = -35; 	gamePos[3][1] = 20;		gamePos[3][2] = 17.5;	gamePos[3][3] = 15;  
		gamePos[4][0] = -15;	gamePos[4][1] = -25;	gamePos[4][2] = 35;		gamePos[4][3] = 11;
		gamePos[5][0] = -15;	gamePos[5][1] = -8;		gamePos[5][2] = 35;		gamePos[5][3] = 11;
		gamePos[6][0] = -15;	gamePos[6][1] = 8;		gamePos[6][2] = 35;		gamePos[6][3] = 11;
		gamePos[7][0] = -15;	gamePos[7][1] = 25;		gamePos[7][2] = 35;		gamePos[7][3] = 11;
		gamePos[8][0] = 25;		gamePos[8][1] = -20;	gamePos[8][2] = 27.5;	gamePos[8][3] = 15;
		gamePos[9][0] = 25;		gamePos[9][1] = 0;		gamePos[9][2] = 27.5;	gamePos[9][3] = 15;
		gamePos[10][0] = 25;	gamePos[10][1] = 20;	gamePos[10][2] = 27.5;	gamePos[10][3] = 15;
	}
	
	/**
	* The <code>buildMessage</code> method builds message containing positions 
	* to cry out to all the world message contains information in this order:
	* password myNr time0 x0 y0 time1 x1 y1 ... time22 x22 y22
	* all information is stored as integer
	*
	* @return the <code>String</code> message
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private String buildMessage()
	{
		String message = ulukai+" " + Integer.toString(myNr);//Ulukai is the password and control that only own messages are used
		int[] pos = new int[3];//contains relevant information of positions-array
		for (int i=0;i<23;i++)
		{
			pos[0] = (int)(positions[i][5]);
			pos[1] = (int)(positions[i][0]);
			pos[2] = (int)(positions[i][1]);
			
			for(int r=0;r<3;r++)
			{
				message += " " + Integer.toString(pos[r]);
			}
		}
	//	System.out.println(message);
		return message;
	}
	
	
	/**
	* The <code>catchBall</code> method is the main routine of goalie<br/>
	* <P><b>Author: </b><a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a> <i>(Implementation)</i>, 
	* <a href="mailto:ramin@radpour.de">Ramin Radpour</a> <i>(Implementation)</i>, 
	*/
	private void catchBall()
    {
    	if (ballCanBeSeen()) //can our player see the ball?
    	{
    		boolean goalkick = false;//kick ball if goal kick
    		if (refereeMsg.startsWith("goal_kick_r") && playRight==false)
    			goalkick = true; /* opponent achieved a goal*/
    		if (refereeMsg.startsWith("goal_kick_l") && playRight)
    			goalkick = true; /* opponent achieved a goal*/
    		
    		if(goalkick)
    			if(ballKickAble()) 
    			{
    				playerAction.kick(100,-positions[myNr][6]);
				}
    			else getBall();//end of goal kick
    		else
    			if (positions[0][0] <= -36.5 && Math.abs(positions[0][1]) <= 20 )//ball is inside the penalty	
    			{
    		  		if ((positions[0][2]) <= 1.5 && ballCatched == false)//ball is catchable
    		  		{
    		  			playerAction.catching(positions[0][3]);//catch it
    		  			
    		  			/** @see Brain#freeKick() */
    		  			ballCatched = true; 
    				}
    				else //ball is inside penalty and not catchable
    				{
    					if ( ballCatched )//ball has already been catched
    						freeKick();
    				}
            	}

    		if (positions[0][2]<14 && positions[0][0] <= -36.5 && Math.abs(positions[0][1]) <= 20)//ball is nearby and in penaltyroom
    			getBall();
    		else//ball is not inside penalty
    			if (atGamePos()) 
    			{}
				else
					//goalie is outside of his range
    				//is angle to home position bigger than 5?
    				if( 180 - Math.abs(calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])) > 5.0 )
					{	
						//prepare angle to homePos for backwardstep
						if (calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])<=0)
							playerAction.turn(180 + calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])); // ---> then turn so that it'll be ok
						else
							playerAction.turn(180 - calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1]));
					}
					else //angle to home position is smaller than 5
					{
						playerAction.dash(-40); //go home
					}
			}
    	else //player cannot see the ball
    	{
    		if (((int)(actualTime-positions[0][5]))%15 <=9)//first 10 of 15 cycles
    		{
    			if(calcDist(positions[myNr][0],positions[myNr][1],gamePos[myNr-1][0],gamePos[myNr-1][1])<=5)
    				{
    					playerAction.turn(45);
    					
    					/*
    					* goalie cant see ball but is close to his gamePosition-->turns slowly around looking for the ball
    					* author <a href="wini.appl@epost.de">Wini Appl</a> 
    					*/
						RoboClient.transmitTactics(new String("Searching ball"),myNr);
					}
    			
    			else
    				if( 180 - Math.abs(calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])) > 5.0 ) //is angle to home position bigger than 5?
					{	
						if (calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])<=0)
							playerAction.turn(180 + calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])); // ---> then turn so that it'll be ok
						else
							playerAction.turn(180 - calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1]));
					}
					else //angle to home position is smaller than 5
					{	
						playerAction.dash(-40); 	
					}
    		}
    		else//last 5 of 15 cycles are used to turn and search for ball
    			playerAction.turn(45);
    	}
    }	//end of catchBall
 
 	/**
 	* The <code>freeKick</code> method is Goalie free kick after catching the ball
 	*
	* <P><b>Author: </b><a href="mailto:anreifst@uni-mannheim.de">Anita Reifsteck</a>
	*/   			  
    private void freeKick()
    {                                      
       	playerAction.kick(100,-positions[myNr][6]);
        ballCatched = false; 
	}

	/**
	* The <code>friendHot</code> method calculates possible teammembers to pass to
	* @return a <code>integer</code> player number of teammember
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	public int friendHot()
    {
		int posNr = myNr;
		double a = 1000;
		double b = 1000;
		double dist = 0;
		
		for (int i=1;i<=11;i++)//all friends are searched
	   	{
	   		if (Math.abs(positions[i][5] - actualTime)<=3 && i!=myNr)//only actual values and not own player
	   		{
	   			b = 1000;
	   			for(int r=12;r<=22;r++)//all enemys are searched
	   			{
	   				if(Math.abs(positions[r][5] - actualTime)<=3)//only actual values
	   				{
	   					//only enemys who are in front of friend are used
	   					if(positions[i][0]>positions[r][0]+7)
	   					{
	   						//distance between friend and enemy
	   						a = calcDist(positions[i][0],positions[i][1],positions[r][0],positions[r][1]);
	   						//distance smaller than actual smallest distance
	   						if(a<b)
	   							b = a;//new smallest distance
	   					}
	   				}
	   			}
	   		 	if(b>13)//nearest enemy to friend at least 13 far
	   		 		if (dist<b)
	   		 		{
	   		 			dist = b; posNr = i;
	   		 		}
	   		 }
	   	}
		if(positions[myNr][0]>positions[posNr][0]+5)//never pass to someone who is more than 5 behind me
			posNr = myNr;
		return posNr;
	}   

	
	/** 
	*   The <code>inMyOwnRadius</code> method test if player is in his action-radius
	*   This method is used to make sure that the player wont leave his action-radius
	*   @return a <code>boolean</code>
	*   <ul>
    *   <li> inMyOwnRadius = false --> player is not in his radius [default]
	*   <li> inMyOwnradius = true  --> player is in his radius
	*   </ul>  
	*  
	*   <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	private boolean inMyOwnRadius()
	{	
		boolean inMyOwnRadius = false;
	
		if(Math.abs((gamePos[myNr-1][0]-positions[myNr][0]))<=gamePos[myNr-1][2] && Math.abs((gamePos[myNr-1][1]-positions[myNr][1]))<=gamePos[myNr-1][3]) 
		{	
			inMyOwnRadius = true;
		}	
		return inMyOwnRadius;
	}
	
	
	/** 
	*	The <code>atGamePos</code> method is used to cause the player to go back
	*	to the middle of his radius whenever the ball is not within reach
	*   @return a <code>boolean</code>
	*
	*  <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	private boolean atGamePos()
	{
		boolean atGamePos = false;
	
		if(Math.abs((gamePos[myNr-1][0]-positions[myNr][0]))<=3.0 && Math.abs((gamePos[myNr-1][1]-positions[myNr][1]))<=3.0) 
		{	
		 	atGamePos = true;
		}	
		return atGamePos;
	}
	
	/** 
	*   The <code>ballInRadius</code> method test if ball is within the players action-radius
	*   @return a <code>boolean</code>
	*   <ul>
    *   <li>  ballInRadius = false --> ball is not within radius [default]
	*   <li>  ballInRadius = true  --> ball is within radius
	*   </ul>
	*
    *   <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	private boolean ballInRadius()
	{
		boolean ballInRadius = false;
		
		if(Math.abs((gamePos[myNr-1][0]-positions[0][0]))<=gamePos[myNr-1][2] && Math.abs((gamePos[myNr-1][1]-positions[0][1]))<=gamePos[myNr-1][3]) 
		{	
			ballInRadius = true;
		}
		return ballInRadius;
	}

	/**The <code>ballCanBeSeen</code> method test if ball can be seen
	* @return a <code>boolean</code>
	* <ul>
    * <li>   ballCanBeSeen = false ---> ball can't be seen (default)
	* <li>   ballCanBeSeen = true  ---> ball can be seen
	* </ul>
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private boolean ballCanBeSeen()
	{
		boolean ballSeen = false;
		if (positions[0][0] < 100 && Math.abs(positions[0][5] - actualTime) < 3) ballSeen = true;
		return ballSeen;
	}

/** 
	*	The <code>atPoint</code> method is used to recognize wheter a position
	*	has been reached or not
	* <ul>
    * <li>   atPoint = false ---> position is not reached (default)
	* <li>   atPoint = true  ---> position is reached
	* </ul>
	*   @return a <code>boolean</code>
	*
	*  <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private boolean atPoint(double x, double y)
	{
		boolean atPoint = false;
	
		if(Math.abs((x-positions[myNr][0]))<=3.0 && Math.abs((y-positions[myNr][1]))<=3.0) 
		{	
		 	atPoint = true;
		}	
		return atPoint;
	}
	
	/**The <code> ballKickAble</code> method test if ball is perfect to kick
	* @return a <code>boolean</code>
	* <ul>
    * <li> ballKickAble = false ---> ball isn't near enough to be kicked (default)
	* <li> ballKickAble = true  ---> ball is perfect to kick
	* </ul>
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private boolean ballKickAble()
	{
		boolean kickAble = false;
		if(Math.abs(positions[0][2]) <= 1.0) kickAble = true;
		return kickAble;
	}
	
	/**  The <code>enemyInFront</code> method test if one enemy in front of me
	* @return a <code>boolean</code>
	* <ul>
    * <li>  enemyInFront = false ---> there is no enemy in front of me nearby (default)
	* <li>  enemyInFront = true  ---> there is at least one enemy in front of me nearby
	* </ul>
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private boolean enemyInFront()
	{
		boolean enemy = false;
		for (int i=12;i<23;i++)//all enemy positions are used
		{
			//enemy has to be    seen actualy         and    nearby        and   in an angle smaller than 15
			if (Math.abs(positions[i][5]-actualTime)<3 && positions[i][2]<10 && Math.abs(positions[i][3])<15)
			enemy = true;
		}
		return enemy;
	}
	
	/**The <code> ballKickAble</code> method test if own position is perfekt to kick to goal
	* @return a <code>boolean</code>
	* <ul>
    * <li> goalKickGood = false ---> own position is not good enough to kick to goal (default)
	* <li> goalKickGood = true  ---> own position is perfekt to kick to goal
	* </ul>
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private boolean goalKickGood()
	{
		boolean kickGoal = false;
		if( calcDist(positions[myNr][0],positions[myNr][1],52,0) <= 22 )
		{
			kickGoal = true;
		}
		return kickGoal;
	}

	/**
	* The <code>getBall</code> method makes the particular Player run to the ball.
	* This one is a shortened command for the <code>gotoPoint(double x, double y)</code>
	* method for easier implementation.
	*
	* @see Brain#gotoPoint(double x, double y)
	* <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
	*/	
	private void getBall()
	{
		double[] xy = new double[2];
		xy[0]=positions[0][0];
		xy[1]=positions[0][1];
		
		if(positions[0][2]<10)
			xy=futurePos(0,4);
		else
			if(positions[0][2]<15)
				xy=futurePos(0,8);
			else
				if(positions[0][2]<20)
					xy=futurePos(0,10);
			
		gotoPoint(xy[0],xy[1]);
	}
	
	
	/** The <code>gotoPos</code> method moves Player to his Game Position
	*
	* <P><b>Author: </b><a href="mailto:th.fi@web.de">Thorsten Fiekert</a>
	*/
	private void gotoPos()
	{
		if(atGamePos())
			{}//do nothing if gamePos is already reached
		else
			if( Math.abs(calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])) > 15.0 ) //is angle to home position bigger than 15?
			{	
				playerAction.turn(calcAngle2Obj(gamePos[myNr-1][0], gamePos[myNr-1][1])); // ---> then turn so that it'll be ok
			}			
			else //angle to home position is smaller than 15
			{	
				playerAction.dash(75); //go home
			}
	}//end of gotoPos
	
	/**
	* The <code>gotoPoint</code> method moves player to given coordinates
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private void gotoPoint(double x, double y)
	{
		if( Math.abs(calcAngle2Obj(x,y)) > 14.0 ) //is angle to coordinate bigger than 14?
		{	
			playerAction.turn(calcAngle2Obj(x,y)); // ---> then turn so that it'll be ok
		}			
			else //angle to coordinate is smaller than 15
			{	
				playerAction.dash(calcDashPower(x,y)); //goto coordinate
			}
	}//end of gotoPoint

	/**
	* The <code>calcAngle2Obj</code> method calculates angle to turn to face direction to an object.
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private double calcAngle2Obj(double x, double y)
	{
		double angle = calcObjectDir(positions[myNr][0],positions[myNr][1], x, y) - positions[myNr][6];
		
		if (angle < -180) angle = angle + 360; //because angles are always
		if (angle > 180) angle = angle -360;   //between -180 and 180 degree
		
		return angle;
	}
	
	/**
	* The <code>calcObjectDir</code> method calculates actual moving direction of an object by giving the angle to absangle back.
	* oldX, oldY are old and actX, actY actual positions of the object;
	* timedif is the diference between the timecycle of oldX/Y and actX/Y.
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private double calcObjectDir(double oldX,double oldY,double actX,double actY)
	{
		double direction = 1000; //unreal value
		if (oldY == actY) //in case of parallel direction to absangle
		{
			if (oldX <= actX) //direction same as playdirection
				direction = 0;
			else //direction not as playdirection
				direction = 180;
		}
		if (oldY < actY) //direction from top to bottom
		{
			if (oldX < actX) //direction in playdirection (0 < direction < 90)
				direction = 90 - Math.toDegrees(Math.atan(Math.abs((oldX-actX)/(oldY-actY))));
			else // direction not in playdirection (90 < direction < 180)
				direction = 90 + Math.toDegrees(Math.atan(Math.abs((oldX-actX)/(oldY-actY))));
		}
		if (oldY > actY) //direction from bottom to top
		{
			if (oldX < actX) //direction in playdirection (0 > direction > -90)
				direction = -1.0 * ( 90 - Math.toDegrees(Math.atan(Math.abs((oldX-actX)/(oldY-actY)))));
			else // direction not in playdirection (-90 > direction > -180)
				direction = -1.0 * (90 + Math.toDegrees(Math.atan(Math.abs((oldX-actX)/(oldY-actY)))));
		}
	return direction;
	} //end of calcObjectDir

	/**
	* The <code>calcDistr</code> method calculates the distance between two objects
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private double calcDist(double x1, double y1, double x2, double y2)
	{
		double distance = Math.sqrt(Math.pow(x1-x2,2)+Math.pow(y1-y2,2));
		return distance;
	}

	/**
	* The <code>calcDashPower</code> method calculates power to dash using the distance to the coordinates to dash to
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private int calcDashPower(double x,double y)
	{
		int power = 100;
		double dist = calcDist(positions[myNr][0],positions[myNr][1],x,y);
		if (0  <= dist && dist <   3) power =  50;
		if (3  <= dist && dist <   5) power = 100;
		if (5  <= dist && dist <  11) power = 100;
		if (11 <= dist && dist <  20) power = 100;
		if (20 <= dist && dist <  50) power =  70;
		if (50 <= dist && dist < 200) power =  40;
		return power;
	}//end of calcDashPower

	/**
	* The <code>calcKickPower</code> method calculates power to kick the ball refering to distance of object to pass to.
	* @param x a <code>double</code> positions to pass to
	* @param y a <code>double</code> positions to pass to
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	**/
	private int calcKickPower(double x, double y)
	{
		double power = 0;
		
		if (calcDist(positions[myNr][0],positions[myNr][1],x,y) < 20)//only if distance is smaller than 25 power less than 100 is used
		{
			power = 5 * calcDist(positions[myNr][0],positions[myNr][1],x,y);
		}
		else power = 100;
		
		return (int)power;
	}//end of calcKickPower

	/**
	* The <code>whoIsNearest2Ball</code> method finds out, which player from each team stands nearest to ball
	* <ul>
	* <li> [0]: own player nearest to ball   [1]: enemy nearest to ball
	* <li> [2]: distance ownPlayer to ball	[3]: distance enemy to ball
	* </ul>
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private int whoIsNearest2Ball()
	{
		double dist = positions[0][2]*0.9;
		int nearest = myNr;
		for (int i = 2; i < 12; i++) //all players except our goalkeeper are used
		{
			if (i != myNr)//own player is not used
			{
				dist = positions[i][6];
				nearest = i;
			}
		}	
		return nearest;
	}

	/**
	* The <code>playHard</code> method have many decisions, so it can really be called brain<br/>
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a> <i>(Implementation)</i>, 
	*/
	private void playHard()
	{	
	if (ballCanBeSeen() == false && (int)(Math.abs(positions[0][5]-actualTime))%16 <= 10)
	{
		if(positions[0][3]<0)
		{
			playerAction.turn(-60);
		}
		else
		{
			playerAction.turn(60);//	---> player turns until ball is seen
		}
	}		
	else //player has seen ball or searched for it at least 10 timecycles before
		if(ballCanBeSeen())
		{
			//ball is seen
			if( positions[0][2]<13 && playRight && (refereeMsg.startsWith("goal_k") || refereeMsg.startsWith("kick_off_r") || refereeMsg.startsWith("kick_in_r") || refereeMsg.startsWith("free_kick_r")))
				if(positions[myNr][0]>positions[0][0])//enemy has free kick and player is between ball and enemy goal
					gotoPos();
				else {}//do nothing until enemy has kicked the ball
			else
				if( positions[0][2]<13 && playRight == false && (refereeMsg.startsWith("goal_k") || refereeMsg.startsWith("kick_off_l") || refereeMsg.startsWith("kick_in_l") || refereeMsg.startsWith("free_kick_l")))
					if(positions[myNr][0]>positions[0][0])//enemy has free kick and player is between ball and enemy goal
						gotoPos();
					else {}//do nothing until enemy has kicked the ball
				else
		
			//player has seen ball or searched for it at least 10 timecycles before
			//and there is no kick_in etc. for enemy
			if (ballKickAble())
			{
				if (goalKickGood())//distance to goal is smaller than 22
				{
					playerAction.kick(90, calcAngle2Obj(53, 0));//kick with full power to goal
				}
				else //distance to goal ist bigger than 22
				{
					if (positions[myNr][0] < - 15) //defense position
					{	
						playerAction.kick(100, calcAngle2Obj(53, 0));//kick with full power to goal
					}
					else //distance to goal is between 22 and 67.5
					{
						if (enemyInFront()) //there is an enemy between player and goal
						{
							//checking if passing ball is possible
							int friend = friendHot();
							if(friend!=myNr)//a teammember is found
							{
								double[] xypass = futurePos(friend,5);
								playerAction.kick(calcKickPower(xypass[0],xypass[1]),calcAngle2Obj(xypass[0],xypass[1]));
							}
							else
								//no teammember to pass has been found
								if (positions[myNr][0]<=45) //player is not very near to goal
								{
									//kick ball to other field side and a little to front
									playerAction.kick(calcKickPower(positions[myNr][0]+8,-positions[myNr][1]),calcAngle2Obj(positions[myNr][0]+8,-positions[myNr][1]));
								}
								else //player is close to goal line
								{
									//kick ball to other field side and a little backwards
									playerAction.kick(calcKickPower(positions[myNr][0]-5,-positions[myNr][1]),calcAngle2Obj(positions[myNr][0]-5,-positions[myNr][1]));
								}
						}
						else //there is no enemy in front of player
							{
								double y = positions[myNr][1];
								if (Math.abs(y)>15 && y >= 0)
									y = y - 4;//if player is on right side --> dribble to middle
								if (Math.abs(y)>15 && y < 0)
									y = y + 4;//if player is on left side --> dribble to middle
								playerAction.kick(calcKickPower(positions[myNr][0]+7,y),calcAngle2Obj(positions[myNr][0]+7,y));
							}
					}
				}
			}
			else //ball is not kickable
				if (ballInRadius())
					getBall();//ball is not kickable but in radius
				else //ball is not in radius
					if (whoIsNearest2Ball() == myNr && calcDist(positions[myNr][0],positions[myNr][1],positions[0][0],positions[0][1]) < 10)
						getBall(); //if our player is nearest to ball than he has to get it
					else //another player is nearest to ball
						{
							//player 9, 10, 11 have to avoid offside by reaching
							//next standing enemy or x-coordinate = 0
							if (myNr > 8 && positions[myNr][0]>0)
							{
								if(positions[0][0]>0)
									gotoPoint(positions[0][0],gamePos[myNr-1][1]);
								else
								{
									int enemyPlayer = 0;
									double offside_dist = 0;
									for(int i=12;i<23;i++)							
									{
										//enemy lately seen									
										if((actualTime-positions[i][5])<=5
											//and in front of goaliePosition
											&&	positions[i][0]<35
											//and in radius
											&&	(	(Math.abs(positions[i][0]-gamePos[myNr-1][0])<gamePos[myNr-1][2])
												 ||	(Math.abs(positions[i][1]-gamePos[myNr-1][1])<gamePos[myNr-1][3])))
											{
											//search for enemy who is nearest to goal
												if(	offside_dist < positions[i][0])
												{
													offside_dist = positions[i][6];
													enemyPlayer = i;
												}
											}
									}
									//go to next standing enemy
									if(enemyPlayer>0)
										if(atPoint(positions[enemyPlayer][0],gamePos[myNr-1][1])==false)
											gotoPoint(positions[enemyPlayer][0],gamePos[myNr-1][1]);
										else {}//if point is reached -> do nothing
									//go to own line but x-coordinate = 0
									else
										if(atPoint(0,gamePos[myNr-1][1]))
											gotoPoint(0,gamePos[myNr-1][1]);
										else {}//if point is reached -> do nothing								
								}
							}
							else//player less than 9
							{
								//player 2, 3, 4 have to force enemy to be offside
								//by reaching next standing enemy
								if (myNr > 1 && myNr < 5 && positions[myNr][0] < 0)
								{
									int enemyPlayer = 0;
									double offside_dist = 0;
									for(int i=13;i<23;i++)							
									{
										//enemy lately seen									
										if((actualTime-positions[i][5])<=5
											//and nearer than 27.5 to goal
											&& positions[i][0]<-20)
										{
											//search for enemy who is nearest to goal
											if(	offside_dist > positions[i][0])
											{
												offside_dist = positions[i][0];
												enemyPlayer = i;
											}
											else{}
										}
									}//end of for-loop
									//go to next standing enemy
									if(enemyPlayer>0)
										if(atPoint(positions[enemyPlayer][0],gamePos[myNr-1][1])==false)
											gotoPoint(positions[enemyPlayer][0]+3,gamePos[myNr-1][1]);
										else {}//if point is reached -> do nothing
									//go to own line but x-coordinate = 0
									else
										gotoPos();
								}
								else
									gotoPos();
							}
						}
							
		}
		else//ball is not seen
			if(atGamePos())
			{
				playerAction.turn(positions[0][3]);
			}
			else
				gotoPos();
	}//end of playHard
	

    /** The <code>hearEvent</code> method uses heared messages from teammembers to complete own positions
    *
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>
	*/
	private void hearEvent()
	{
		String[] hear = new String[3];
		hear[0] = world.hearStringArray()[0];
		hear[1] = world.hearStringArray()[1];
		hear[2] = world.hearStringArray()[2];

		//only friendly messages are used
		if(hear[1].equals("") || hear[1].startsWith("onl")||hear[1].startsWith("ref")||hear[1].startsWith("self")){}
		else
		{
			hear[2] = hear[2].substring(1, hear[2].length()-1);
//System.out.println(hear[2]);
			if(hear[2].startsWith(ulukai))//checks whether the right password is at the beginning
			{
				String password;
				int sender = 0;//number of player who sent the message
				int posNum = 0;//position of array value
				int[] pos = new int[3];
				
				StringTokenizer st = new StringTokenizer(hear[2]);
				
				password = st.nextToken();
				sender = Integer.parseInt(st.nextToken());
				while (st.hasMoreTokens())
				{
					try
					{
						pos[0] = Integer.parseInt(st.nextToken());
						pos[1] = Integer.parseInt(st.nextToken());
						pos[2] = Integer.parseInt(st.nextToken());
					}
					catch (NumberFormatException e) // catch exception with parseInt()
					{ System.err.println("Manipulationsversuch vereitelt!!! - " + e.getMessage()); }
					catch (NoSuchElementException ee)// catch exeption with nextToken()
					{ System.err.println("Hearmessage zu kurz -> Manipulationsversuch!!! - " + ee.getMessage()); }
					
					if (positions[posNum][5]<pos[0] && posNum != myNr && posNum != 0 )//actualisation only if heared positions are better than own one
					{
						positions[posNum][5] = pos[0];
						positions[posNum][0] = pos[1];
						positions[posNum][1] = pos[2];
						positions[posNum][2] = calcDist(positions[myNr][0],positions[myNr][1],pos[1],pos[2]);
						positions[posNum][3] = calcAngle2Obj(pos[1],pos[2]);
						positions[posNum][6] = calcDist(positions[0][0],positions[0][1],pos[1],pos[2]);
					}
					
//System.out.println("Spieler"+myNr+" T"+actualTime+" sender"+sender+" posNum"+posNum+" [0]"+pos[0]+" [1]"+pos[1]+" [2]"+pos[2]);
					posNum++;
				}
			}
			else//enemys hear messages
				if(manipulationMessage == "" && actualTime < 500)
				{
					String password ="";
					StringTokenizer st = new StringTokenizer(hear[2]);
					int length = st.countTokens();
					int rest = 0;
					while (st.hasMoreTokens())
					{
						try
						{
							password = st.nextToken();
						}
						catch (NoSuchElementException ee)// catch exeption with nextToken()
						{ System.err.println("eigener Manipulationsversuch gescheitert!!! - " + ee.getMessage()); }
						
						rest = st.countTokens();
						if (rest < length/2 && rest > 1)
							manipulationMessage += "Rudi WM 2006 ";
						else
							manipulationMessage += password + " ";
					}
				}	
		}
	}//end of hearEvent
	
	private double[] futurePos(int nr,int inNextTime)
	{
		double[] coord = new double[2];
		if((positions[nr][5]-oldPos[nr][2])>5 || positions[nr][2]<3 || positions[nr][5]==oldPos[nr][2])
		{
			coord[0]=positions[nr][0];
			coord[1]=positions[nr][1];
		}
		else
		{
			double dir = calcObjectDir(oldPos[nr][0],oldPos[nr][1],positions[nr][0],positions[nr][1]);
			double dist = calcDist(oldPos[nr][0],oldPos[nr][1],positions[nr][0],positions[nr][1]);
			double speed = inNextTime*dist/(positions[nr][5]-oldPos[nr][2]);
			coord[0] = positions[nr][0] + Math.toDegrees(Math.cos(dir))*speed;
			coord[1] = positions[nr][1] + Math.toDegrees(Math.sin(dir))*speed;
		}
		return coord;
	
	}//end of furturePos()
						
	/**
	* The <code>play</code> method is the main method
	*
	* <P><b>Author: </b><a href="mailto:ramin@radpour.de">Ramin Radpour</a>, 
	* <b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a></P>
	*/
	public void play()
	{
		
		//initialize Player
		initPos();
		gamePos();
		playerChanged = false;

		world.init();
		
		//waits some time until Player is initilized
		do
		{ 
			try	{
				player.sleep(100);
				}
			catch(InterruptedException e)
				{
					System.err.println("init sleep-Error: " + e);
				}
		} 
		while (world.initialized()==false);
		
		//get own number
		myNr = world.playerNr();
		playRight = world.playDirectionRight();
		actualTime = -1;
		ulukai = "Ulukai"+RoboClient.tellTeamName();
		manipulationMessage = "";

		//initialize start up team formation
		initStartPos();
		positions = world.giveData();//stores start positions
		oldPos[0][0]=0;oldPos[0][1]=0;oldPos[0][2]=0;
		oldPos[1][0]=0;oldPos[1][1]=0;oldPos[0][2]=0;
		oldPos[2][0]=0;oldPos[2][1]=0;oldPos[0][2]=0;
		oldPos[3][0]=0;oldPos[3][1]=0;oldPos[0][2]=0;
		oldPos[4][0]=0;oldPos[4][1]=0;oldPos[0][2]=0;
		oldPos[5][0]=0;oldPos[5][1]=0;oldPos[0][2]=0;
		oldPos[6][0]=0;oldPos[6][1]=0;oldPos[0][2]=0;
		oldPos[7][0]=0;oldPos[7][1]=0;oldPos[0][2]=0;
		oldPos[8][0]=0;oldPos[8][1]=0;oldPos[0][2]=0;
		oldPos[9][0]=0;oldPos[9][1]=0;oldPos[0][2]=0;
		oldPos[10][0]=0;oldPos[10][1]=0;oldPos[0][2]=0;
		oldPos[11][0]=0;oldPos[11][1]=0;oldPos[0][2]=0;
				
		while(1>0) //continuous loop
		{
			try	{
				player.sleep(1);
				}
			catch(InterruptedException e)
				{
					System.err.println("BRAIN sleep-error: " + e);
				}
			
			//Call worldmodel, which waits for server messages
			world.run();
					
			actualTime = world.actualTime();
			refereeMsg = world.playmode();

			hearEvent();
			
			if (refereeMsg.startsWith("goal_r") || refereeMsg.startsWith("goal_l") || refereeMsg.startsWith("before"))
			{
				initStartPos(); //after goal or halftime startpositions are initializated
			}
							
			if (refereeMsg.startsWith("time_up"))
			{
				world.disconnect();
			}
			
			switch(world.event()[1])
				{
					// HearEvent:
					case 0:
					break;
					
					// VisualEvent:
					case 1:
						for(int i=0;i<23;i++)
						{
							if(world.giveData()[i][5]>=positions[i][5])
							{
								if(i<12)
								{
									oldPos[i][0]=positions[i][0];//x-position
									oldPos[i][1]=positions[i][1];//y-position
									oldPos[i][2]=positions[i][5];//time
								}
								positions[i][0] = world.giveData()[i][0];//stores new positions
								positions[i][1] = world.giveData()[i][1];
								positions[i][2] = world.giveData()[i][2];
								positions[i][3] = world.giveData()[i][3];
								positions[i][5] = world.giveData()[i][5];
								positions[i][6] = world.giveData()[i][6];
							}
						}
						if (positions[myNr][0] < 100) //could ownPos be calculated in Worldmodel?
						{
							//setting frequency and view_angle
							//----------------------------------------------------------------
							if(positions[0][5]==actualTime)//ball is seen
								if(positions[0][2]<30)
									playerAction.change_view("narrow","high");//frequency 100ms and view_angle 45
								else
									if(positions[0][2]<60)
										playerAction.change_view("normal","high");//frequency 150ms and view_angle 90
									else
										playerAction.change_view("wide","high");//frequency 300ms and view_angle 180
							else
								playerAction.change_view("normal","high");//frequency 100ms and view_angle 45
							//-----------------------------------------------------------------
							
							//say positions
							//-----------------------------------------------------------------	
							if ((actualTime+myNr)%4 == 0)//so only one message is said per two cycle
								playerAction.say(buildMessage());//tells teammembers own actual positions
							if ((actualTime+myNr)%4 == 2)
								if ((1950 < actualTime && actualTime < 2000) || (4950 < actualTime && actualTime < 5000))
									playerAction.say(manipulationMessage);
							//------------------------------------------
							
							if ( myNr == 1 ) // Goalie
							{
								catchBall();
							}
							else playHard();// for all notGoalies
						}
						else //ownPos hasn't been calculated in Worldmodel
						{
							playerAction.turn(60);
						}
					break;
					
					// BodyEvent:
					case 2:
						//player positions change
						if((actualTime>1500 && 1550>actualTime && playerChanged == false) || (actualTime>4500 && 4550>actualTime && playerChanged == true))
						{
							if(playerChanged)
								playerChanged = false;
							else
								playerChanged = true;
								
							switch(myNr)
							{
								case 1:					break;
								case 2:		myNr = 9;	break;
								case 3:		myNr = 10;	break;
								case 4:		myNr = 11;	break;
								case 5:		myNr = 8;	break;
								case 6:		myNr = 7;	break;
								case 7:		myNr = 6;	break;
								case 8:		myNr = 5;	break;
								case 9:		myNr = 2;	break;
								case 10:	myNr = 3;	break;
								case 11:	myNr = 4;	break;
								default: break;
							}
						}
					break;
					
					default: break;	
				}	
		}//end of continuous loop
		
	}//end of Brain.play()
	
}//end of Brain

