import java.io.*;
import java.net.*;
/**
 * The <code>Communication</code> class takes care of the server-client
 * UDP-Packet-transmission. This one is called by each <code>Player</code>-class.
 *
 * @see Player
 * @author <a href="wini.appl@epost.de">Wini Appl</a>
 */
public class Communication
{
	/**initializes a new Datagram Socket-class*/
    private DatagramSocket datagramSocket;
    
    /**this java-own class represents an Internet Protocol-address*/
    private InetAddress serverIP;
    private int serverPort;

    /**
     * Constructor of another <code>Communication</code> instance; handles
     * transfer errors.<br/>
     *
     * <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
     *
     * @param serverName Name of the RoboCupServer one wants to connect to.
     * @param serverPort Portnumber of the RoboCupServer one wants to connect to.
     */
    public Communication(String serverName, int serverPort) 
    {
        try 
        {
            this.datagramSocket = new DatagramSocket();
            this.serverIP = InetAddress.getByName(serverName);
            this.serverPort = serverPort;	
        } 
        /**exits runtime due to communication problems with a status-number (1 or 2)*/
        catch (SocketException e) 
        {
            System.err.println("<Datagram Socket could not be initialized>");
            System.exit(1);
        } 
        catch (UnknownHostException e) 
        {
            System.err.println("<No Server IP available>");
            System.exit(2);
        }
    }

    /**
    * The <code>Send</code>-method converts any incoming input-string into a 
    * byte-array and sends it to the server.<br/>
    *
    * <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
    *
    * @param input sends the parsed String to aboves Server
    * @see Communication#Communication(String serverName, int serverPort) 
    * @see Parser
    */
    public void send(String input) 
    {
        try 
        {
        	/**the <code>DatagramPacket</code> needs byte-arrays to work with*/
            byte[] input2server = input.getBytes();
            DatagramPacket packet = new DatagramPacket(input2server, input2server.length, serverIP, serverPort);
            datagramSocket.send(packet);
        } 
        catch (IOException e) 
        {
            System.err.println("<Datagram Packet could not be sent>");
        }
    }

    /**
     * The <code>Receive</code>-method fetches data from the server and passes
     * it on to the <code>Worldmodel</code>.<br/>
     *
     * <P><b>Author: </b><a href="mailto:wini.appl@epost.de">Wini Appl</a></P>
     *
     * @return Received Data is getting fetched by the Parser for further treatment.
     * @see Parser
     */
    public String receive() 
    {
    	/**the <code>DatagramPacket</code> needs byte-arrays to work with. This one is 2048 Characters big*/
        byte[] server2client = new byte[2048];
        DatagramPacket packet = new DatagramPacket(server2client, server2client.length);
        try 
        {
        	/**blocks all operations until another message arrives from the server*/
            datagramSocket.receive(packet);
        } 
        catch (IOException e) 
        {
            System.err.println("<Datagram Packet could not be received>");
        }
        /**gets the Port-number of the server the <code>Communication</code>-class is connected to*/
        serverPort = packet.getPort();
        /**trims the received input-String by replacing the last Character.*/
        String output = new String(packet.getData()).substring(0, packet.getLength()-1)+" ";
        return output;
    }
}

