import java.util.*;
/**
 * The <code>Parser</code> class is used to divide the input-stream from
 * the communication-module into tokens. Those are being built up to 
 * a <code>Vector</code> containing <code>VisualObject</code> objects,
 * a <code>String</code> array or a <code>double</code> array,
 * depending on what kind of information the parser module receives.
 *
 * @author <a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 */
public class Parser
{
	/** The Communication object */
	private Communication communication;
	/** The VisualObject object */
	private VisualObject visualObject;
	/** current time */
	private int currentTime;
		
	/**
    * Creates a new <code>Parser</code> instance.
    *
    * @param communication a <code>Communication</code> value defining the
    * communication link to the server via which the commands can be
    * sent.
    * @see Communication
    */
	public Parser(Communication communication)
	{
		this.communication = communication;
		currentTime = -1000;
	}

     /**
     *
     * The <code>tokenizeHearevent</code> method takes 
     * only care of incoming hear messages (see Users Manual 4.2.2).
	 *
   	 * @param hearmsg a <code>String</code> which contains the hear message
   	 * 
	 * @return returns a <code>String</code> array with following content:
	 * <ul> 
	 * <li>[0]: time
	 * <li>[1]: sender { "referee" | "self" | direction | "online_coach_left" | "online_coach_right"}
	 * <li>[2]: message
	 * </ul>
	 * <br>
	 * time can be converted to <code>int</code>
	 * direction can be converted to <code>double</code>
   	 *
   	 * <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
	 */	
	private String[] tokenizeHearevent(String hearmsg)
	{
		String[] hearString = new String[3];
		
		/*
     	* The whole stream gets cut at front and back so unnecessary 
     	* elements are being removed. The resulting hearmsg String gets tokenized.
		*/
		hearmsg = hearmsg.substring(6, hearmsg.length()-2);

		StringTokenizer st = new StringTokenizer(hearmsg);
		// time
		if (st.hasMoreTokens())
		{
			hearString[0]=st.nextToken();
			try
			{
				// set currentTime
				currentTime = Integer.parseInt(hearString[0]);
			} catch (NumberFormatException e) // catch exception with paseInt()
				{ System.err.println("exception: " + e.getMessage()); }
		}
		
		// sender
		if (st.hasMoreTokens()) hearString[1]=st.nextToken();
		// message
		if (st.hasMoreTokens()) hearString[2]=st.nextToken();
		while (st.hasMoreTokens()) hearString[2]+= " "+st.nextToken();

		return hearString; // returns bodyArray to parse()
	
	} // end of private String[] tokenizeHearevent(String hearmsg)
	
    
     
     /**
     *
     * The <code>tokenizeVisualevent</code> method takes 
     * only care of incoming see messages.
     *
 	 * @param hearmsg a <code>String</code> which contains the see message
 	 *
     * @return A <code>Vector</code> containing the perceived and processed 
     * <code>VisualObject</code> objects.
     * @see Vector
 	 *
 	 * <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
 	 */
	private Vector tokenizeVisualevent(String seemsg)
	{
		int timecyclus=0;
		int id[] = new int[2];
		id[1]=1;
		String objectName="";
		String teamName="";
		int uniformNr=0;
		boolean goalie=false;
		double direction=1000;
		double distance=1000;
		double distChange=1000;
		double dirChange=1000;
		double headFaceDir=1000;
		double bodyFaceDir=1000;
		
		Vector vector = new Vector();
		String token = "";
		String subStr = "";
	   	/*
	   	* The "Klammer"-counter determines, at which point
	   	* of the stream the parameters for the visual object begin or if another
	   	* visual object is being stated.
	   	*/
		int klammer = 0;
		int counter = 0;
	 	/*
     	* The whole stream gets cut at front and back so unnecessary 
     	* elements are being removed. The resulting seemsg String gets tokenized.
		* Example for seemsg: "(see 0 ((f p r t) 42.5 23))"
		*/
		seemsg = seemsg.substring(5, seemsg.length()-2);

     	// Creates a new StringTokenizer instance.
		StringTokenizer st = new StringTokenizer(seemsg);

     	try {
     	if (st.hasMoreTokens()) timecyclus = Integer.parseInt(st.nextToken());
     	} catch (NumberFormatException e) // catch exception with paseInt()
			{ System.err.println("exception: " + e.getMessage()); }
			
     	currentTime = timecyclus; // set currentTime

     	// Adds time and ID 1 to vector
    	id[0]=currentTime;
     	vector.add(0,id);  	
		/*
		* 	hasMoreTokens tests if there are more tokens available
		*	from this tokenizer's String.
		*/
     	while (st.hasMoreTokens())
     	{
         	token = st.nextToken();
         	/*
         	* This method toCharArray, taken from the CharArrayWriter,
         	* returns an array of chars copied from the tokenized input data.
         	* and writes it into the char-array called tokenSub.
         	*/
         	char[] tokenSub = token.toCharArray();
         	/* 
         	* Tests, if tokenSub makes a semantic stop and increases/decreases
         	* the "Klammer" counter. Returns the coherent subString.
         	*/
         	for (int i=0; i < tokenSub.length; i++)
         	{
         		if (tokenSub[i] == '(') klammer++;
         		else if (tokenSub[i] == ')') klammer--;
         		else subStr = subStr + tokenSub[i];
         	}

         	if (klammer < 2)
         	{
         		try
         		{
         		switch (counter)
         		{
         			case 0: // Tests, if object is a player
         					if (subStr.startsWith("p"))
         					{ 	
         						// example: p"Teamname"1goalie
         						String sep = "" + '"'; // trick to seperate the teamname
								// seperates the objectname
								StringTokenizer playerst = new StringTokenizer(subStr, sep);
								objectName = playerst.nextToken();
								// Get the teamname
								if (playerst.hasMoreTokens()) teamName= playerst.nextToken();
								String rest = "";
								// Do we have the UniformNumber?
								if (playerst.hasMoreTokens()) rest=playerst.nextToken();
								// Sets the UnifromNr, if the Uniformnumber is smaller than 10
								if (rest.length() == 1) uniformNr = Integer.parseInt(rest);
								// Sets the UniformNr, if the Uniformnumber is 10 or 11
								if (rest.length() > 1)
									{
										if ((rest.charAt(1)== '0') | (rest.charAt(1)=='1'))
											{ uniformNr = Integer.parseInt(rest.substring(0,2)); }
									}
								// If the Player is a goalie the boolean goalie is set to true
								if (rest.length() > 2)
									{	rest=rest.substring(rest.length()-6);
										if (rest.equals("goalie")) goalie=true;
									}
         					}
         					        					
         					// if obect is not a player
         					else objectName = subStr;
         					
         					break;
         					
         			case 1: // Sets the distance
           					distance = Double.parseDouble(subStr); break;
         			case 2: // Sets the direction
         					direction = Double.parseDouble(subStr); break;
         			case 3: // Sets the changed distance
         					distChange = Double.parseDouble(subStr); break;
         			case 4: // Sets the changed direction
         					dirChange = Double.parseDouble(subStr); break;
         			case 5: // Sets the head face direction
         					headFaceDir = Double.parseDouble(subStr); break;
         			case 6:	// Sets the body face direction
         					bodyFaceDir = Double.parseDouble(subStr); break;

         		}
				} catch (NumberFormatException e) // catch exception with paseDouble()
				 	{ System.err.println("exception: " + e.getMessage()); }

         		counter++;
         		
         		// Recognizes a beginning new <code>visualObject</code>.
         		if (klammer==0)
         		{

         			if (direction == 1000) 
         			/*
         			* if ObjInfo is like "(ObjName Direction)"
         			* then  direction is saved in distance..
         			*/
         			{ 
         				direction = distance;
         				distance = 1000;
         			}

         			// create a new VisualObject
         			VisualObject vo = new VisualObject(timecyclus, objectName, teamName, uniformNr, goalie, direction, distance, distChange, dirChange, headFaceDir, bodyFaceDir);
         			// Insert VisualObject into vector
         			vector.addElement(vo);
         			
         			// reset all variables to initialisation value
         			counter = 0;
         			objectName="";
					teamName="";
					uniformNr=0;
					goalie=false;
					direction=1000;
					distance=1000;
					distChange=1000;
					dirChange=1000;
					headFaceDir=1000;
					bodyFaceDir=1000;
         		}	
       			// reset subStr
         		subStr="";
         	}
     	}

		return vector; // returns vector [to parse()]
	} // end of private Vector tokenizeVisualevent(String seemsg)
	

     
     /**
     *
     * The <code>tokenizeBodyevent</code> method takes 
     * only care of incoming body_sense messages (see Users Manual 4.2.2).
   	 *
   	 * @param bodymsg a <code>String</code> which contains the body_sense message
	 * @return returns a double array with following content:
	 * <ul>
	 * <li>[0]: time
	 * <li>[1]: view_mode(1){ 0="high" | 1="low" }
	 * <li>[2]: view_mode(2){ 0="narrow" | 1="normal" | 2="wide" }
	 * <li>[3]: stamina(1)	( Stammina )
	 * <li>[4]: stamina(2)	( Effort )
	 * <li>[5]: speed(1)	( AmountOfSpeed )	
	 * <li>[6]: speed(2)	( DirectionOfSpeed )[-180 ~ 180 degrees]
	 * <li>[7]: head-angle	( HeadAngle )		[-180 ~ 180 degrees]
	 * <li>[8]: kick		( KickCount ) 		[positive integer]
	 * <li>[9]: dash		( DashCount ) 		[positive integer]
	 * <li>[10]: turn		( TurnCount ) 		[positive integer]
	 * <li>[11]: say		( SayCount ) 		[positive integer]
	 * <li>[12]: turn_neck	( TurnNeckCount )	[positive integer]
	 * <li>[13]: catch		( CatchCount )		[positive integer]
	 * <li>[14]: move		( MoveCount )		[positive integer]
	 * <li>[15]: change_view ( ChangeViewCount )[positive integer]
	 * </ul>
	 * (so all numbers are doubles, but some can be converted to int)
	 *
	 * <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
	 */

	private double[] tokenizeBodyevent(String bodymsg)
	{
		double[] bodyArray = new double[16];
		String[] bodyString = new String[16];
		String token = "";
		int tokenCount = 0;
		
		// initialize bodyArray with -1000 and bodyString with no content ("")
		for (int i=0; i<16; i++)
		{
			bodyArray[i] = -1000;
			bodyString[i] = "";
		}
		
		StringTokenizer st = new StringTokenizer(bodymsg);
		
		// divide bodymsg and write everything into bodyString (String array)
		while (st.hasMoreTokens()) 
		{
			token = st.nextToken();
			if ( token.startsWith("(")==false )
			{
				// don't take care of "))"
				if (token.endsWith("))"))
				{ token = token.substring(0,token.length()-2); }
				// don't take care of ")"
				if (token.endsWith(")"))
				{ token = token.substring(0,token.length()-1); }
				try
				{
					bodyString[tokenCount] = token;
				} catch (ArrayIndexOutOfBoundsException e)
				 	{ System.err.println("exception: " + e.getMessage()); }

				tokenCount++;
			}
		} //end of dividing bodymsg
			
		// Convert bodyString[](String array) to bodyArray[] (double array)
		try {
		// time
		bodyArray[0]=Double.parseDouble(bodyString[0]);
		currentTime = (int)bodyArray[0]; // set currentTime
		
		// view_mode(1)
		if (bodyString[1].equals("high")) bodyArray[1]=0;
		if (bodyString[1].equals("low")) bodyArray[1]=1;
		
		// view_mode(2)
		if (bodyString[2].equals("narrow")) bodyArray[2]=0;
		if (bodyString[2].equals("normal")) bodyArray[2]=1;
		if (bodyString[2].equals("wide")) bodyArray[2]=2;
		
		// rest (bodyArray[3]~bodyArray[15]
		for (int i=3; i<16; i++) bodyArray[i]=Double.parseDouble(bodyString[i]);
		} catch (NumberFormatException e) // catch exception with paseDouble()
		 	{ System.err.println("exception: " + e.getMessage()); }


		return bodyArray; // returns bodyArray [to parse()]
	} // end of private double[] tokenizeBodyevent(String bodymsg)
	
	
	

    /**
    *
	* Parse is the main method of the parser. It waits for a new message and
	* starts parsing this message depending on what kind of message it is.
	* @return A <code>Vector</code> with following content:
	* <ol>
	* <li> <b>see message: </b>several <code>VisualObject</code> objects
	* <li> <b>body_sense message: </b>a <code>double</code> array
	* <li> <b>heare message: </b>a <code>String</code> array.
    * </ol>
    *
    * <P><b>Author: </b><a href="mailto:pficher@uni-mannheim.de">Patrick Ficher</a>
	*/	
	public Vector parse()
	{	
		// this is the vector which is returned to worldmodel
		Vector vector = new Vector();
		// id is needed to tell worldmodel what kind of message arrived
		int[] id = new int[2];
		id[1]=3;
		String msg = "";
        while (msg == "")
        {
            /*
            * Receives server messages from communication
            * module. Blocks until any message arrives.
            */
            msg = communication.receive();
    		
    		// hear message min. format: (hear 0 1)
    		if ( (msg.startsWith("(hear"))& (msg.length() > 9) )
    		{
    			id[0] = currentTime;
    			id[1] = 0;
    			/* Adds time, ID 0 and the String[] array which is returned by
    			* 	tokenizeHearevent(String msg) to vector
    			*/
    			vector.add(0,id);
    			vector.add(tokenizeHearevent(msg));
    			
    		}
			
			// see message min. format: (see 0 ((B)))
    		if ( (msg.startsWith("(see")) & (msg.length() > 12) )
    		{
				/* Time, ID 1 and the VisualObject objects 
    			*	are added in tokenizeVisualevent(String msg) to vector
    			*/
    			vector = tokenizeVisualevent(msg);
    		}

    		
    		// "sense_body"-message
    		if ( (msg.startsWith("(sense_body"))& (msg.length() > 160) )
    		{
    			id[0] = currentTime;
    			id[1] = 2;
    			/*	Adds time, ID 2 and the double[] array which is returned by
    			* 	tokenizeHearevent(String msg) to vector
    			*/
    			vector.add(0,id);
    			vector.add(tokenizeBodyevent(msg));
    		}
    	}// end of while
    	
    	return vector; // returns vector [to worldmodel]

    } //end of public Vector parse()
} // end of class Parser